@@ -78,13 +78,14 @@ SPIRVExtInst *SPIRVToLLVMDbgTran::getDbgInst(const SPIRVId Id) {
   SPIRVEntry *E = BM->getEntry(Id);
   if (isa<OpExtInst>(E)) {
     SPIRVExtInst *EI = static_cast<SPIRVExtInst *>(E);
-    if (EI->getExtSetKind() == SPIRV::SPIRVEIS_Debug)
+    if (EI->getExtSetKind() == SPIRV::SPIRVEIS_Debug ||
+        EI->getExtSetKind() == SPIRV::SPIRVEIS_OpenCL_DebugInfo_100)
       return EI;
   }
   return nullptr;
 }
 
-StringRef SPIRVToLLVMDbgTran::getString(const SPIRVId Id) {
+const std::string &SPIRVToLLVMDbgTran::getString(const SPIRVId Id) {
   SPIRVString *String = BM->get<SPIRVString>(Id);
   assert(String && "Invalid string");
   return String->getStr();
@@ -113,8 +114,7 @@ SPIRVToLLVMDbgTran::transCompileUnit(const SPIRVExtInst *DebugInst) {
 
   using namespace SPIRVDebug::Operand::CompilationUnit;
   assert(Ops.size() == OperandCount && "Invalid number of operands");
-  M->addModuleFlag(llvm::Module::Warning, "Dwarf Version",
-                   Ops[DWARFVersionIdx]);
+  M->addModuleFlag(llvm::Module::Max, "Dwarf Version", Ops[DWARFVersionIdx]);
   SPIRVExtInst *Source = BM->get<SPIRVExtInst>(Ops[SourceIdx]);
   SPIRVId FileId = Source->getArguments()[SPIRVDebug::Operand::Source::FileIdx];
   std::string File = getString(FileId);
@@ -228,7 +228,14 @@ SPIRVToLLVMDbgTran::transTypeComposite(const SPIRVExtInst *DebugInst) {
   DIFile *File = getFile(Ops[SourceIdx]);
   unsigned LineNo = Ops[LineIdx];
   DIScope *ParentScope = getScope(BM->getEntry(Ops[ParentIdx]));
-  uint64_t Size = BM->get<SPIRVConstant>(Ops[SizeIdx])->getZExtIntValue();
+
+  uint64_t Size = 0;
+  SPIRVEntry *SizeEntry = BM->getEntry(Ops[SizeIdx]);
+  if (!(SizeEntry->isExtInst(SPIRVEIS_Debug, SPIRVDebug::DebugInfoNone) ||
+        SizeEntry->isExtInst(SPIRVEIS_OpenCL_DebugInfo_100,
+                             SPIRVDebug::DebugInfoNone))) {
+    Size = BM->get<SPIRVConstant>(Ops[SizeIdx])->getZExtIntValue();
+  }
 
   uint64_t Align = 0;
   DIType *DerivedFrom = nullptr;
@@ -679,10 +686,10 @@ SPIRVToLLVMDbgTran::transTemplateParameter(const SPIRVExtInst *DebugInst) {
   if (!getDbgInst<SPIRVDebug::DebugInfoNone>(Ops[ValueIdx])) {
     SPIRVValue *Val = BM->get<SPIRVValue>(Ops[ValueIdx]);
     Value *V = SPIRVReader->transValue(Val, nullptr, nullptr);
-    return Builder.createTemplateValueParameter(Context, Name, Ty,
+    return Builder.createTemplateValueParameter(Context, Name, Ty, false,
                                                 cast<Constant>(V));
   }
-  return Builder.createTemplateTypeParameter(Context, Name, Ty);
+  return Builder.createTemplateTypeParameter(Context, Name, Ty, false);
 }
 
 DINode *SPIRVToLLVMDbgTran::transTemplateTemplateParameter(
