@@ -51,6 +51,7 @@
 #include "llvm/ADT/StringSwitch.h"
 #include "llvm/Bitcode/BitcodeWriter.h"
 #include "llvm/IR/IRBuilder.h"
+#include "llvm/IR/IntrinsicInst.h"
 #include "llvm/Support/CommandLine.h"
 #include "llvm/Support/Debug.h"
 #include "llvm/Support/ErrorHandling.h"
@@ -142,7 +143,7 @@ std::string mapLLVMTypeToOCLType(const Type *Ty, bool Signed) {
   }
   if (auto VecTy = dyn_cast<VectorType>(Ty)) {
     Type *EleTy = VecTy->getElementType();
-    unsigned Size = VecTy->getVectorNumElements();
+    unsigned Size = VecTy->getNumElements();
     std::stringstream Ss;
     Ss << mapLLVMTypeToOCLType(EleTy, Signed) << Size;
     return Ss.str();
@@ -285,7 +286,7 @@ bool isSPIRVType(llvm::Type *Ty, StringRef BaseTyName, StringRef *Postfix) {
 Function *getOrCreateFunction(Module *M, Type *RetTy, ArrayRef<Type *> ArgTypes,
                               StringRef Name, BuiltinFuncMangleInfo *Mangle,
                               AttributeList *Attrs, bool TakeName) {
-  std::string MangledName = Name;
+  std::string MangledName{Name};
   bool IsVarArg = false;
   if (Mangle) {
     MangledName = mangleBuiltin(Name, ArgTypes, Mangle);
@@ -350,7 +351,7 @@ std::string decorateSPIRVFunction(const std::string &S) {
   return std::string(kSPIRVName::Prefix) + S + kSPIRVName::Postfix;
 }
 
-std::string undecorateSPIRVFunction(const std::string &S) {
+StringRef undecorateSPIRVFunction(StringRef S) {
   assert(S.find(kSPIRVName::Prefix) == 0);
   const size_t Start = strlen(kSPIRVName::Prefix);
   auto End = S.rfind(kSPIRVName::Postfix);
@@ -437,11 +438,11 @@ std::string getPostfixForReturnType(const Type *PRetTy, bool IsSigned) {
          mapLLVMTypeToOCLType(PRetTy, IsSigned);
 }
 
-Op getSPIRVFuncOC(const std::string &S, SmallVectorImpl<std::string> *Dec) {
+Op getSPIRVFuncOC(StringRef S, SmallVectorImpl<std::string> *Dec) {
   Op OC;
   SmallVector<StringRef, 2> Postfix;
-  std::string Name;
-  if (!oclIsBuiltin(S, &Name))
+  StringRef Name;
+  if (!oclIsBuiltin(S, Name))
     Name = S;
   StringRef R(Name);
   R = dePrefixSPIRVName(R, Postfix);
@@ -463,7 +464,7 @@ bool getSPIRVBuiltin(const std::string &OrigName, spv::BuiltIn &B) {
 
 // Enqueue kernel, kernel query, pipe and address space cast built-ins
 // are not mangled.
-bool isNonMangledOCLBuiltin(const StringRef &Name) {
+bool isNonMangledOCLBuiltin(StringRef Name) {
   if (!Name.startswith("__"))
     return false;
 
@@ -471,22 +472,19 @@ bool isNonMangledOCLBuiltin(const StringRef &Name) {
          isPipeOrAddressSpaceCastBI(Name.drop_front(2));
 }
 
-bool oclIsBuiltin(const StringRef &Name, std::string *DemangledName,
-                  bool IsCpp) {
+// Demangled name is a substring of the name. The DemangledName is updated only
+// if true is returned
+bool oclIsBuiltin(StringRef Name, StringRef &DemangledName, bool IsCpp) {
   if (Name == "printf") {
-    if (DemangledName)
-      *DemangledName = Name;
+    DemangledName = Name;
     return true;
   }
   if (isNonMangledOCLBuiltin(Name)) {
-    if (DemangledName)
-      *DemangledName = Name.drop_front(2);
+    DemangledName = Name.drop_front(2);
     return true;
   }
   if (!Name.startswith("_Z"))
     return false;
-  if (!DemangledName)
-    return true;
   // OpenCL C++ built-ins are declared in cl namespace.
   // TODO: consider using 'St' abbriviation for cl namespace mangling.
   // Similar to ::std:: in C++.
@@ -503,12 +501,12 @@ bool oclIsBuiltin(const StringRef &Name, std::string *DemangledName,
     size_t Len = 0;
     Name.substr(DemangledNameLenStart, Start - DemangledNameLenStart)
         .getAsInteger(10, Len);
-    *DemangledName = Name.substr(Start, Len);
+    DemangledName = Name.substr(Start, Len);
   } else {
     size_t Start = Name.find_first_not_of("0123456789", 2);
     size_t Len = 0;
     Name.substr(2, Start - 2).getAsInteger(10, Len);
-    *DemangledName = Name.substr(Start, Len);
+    DemangledName = Name.substr(Start, Len);
   }
   return true;
 }
@@ -549,8 +547,8 @@ void eraseSubstitutionFromMangledName(std::string &MangledName) {
   }
 }
 
-ParamType lastFuncParamType(const std::string &MangledName) {
-  auto Copy = MangledName;
+ParamType lastFuncParamType(StringRef MangledName) {
+  std::string Copy(MangledName);
   eraseSubstitutionFromMangledName(Copy);
   char Mangled = Copy.back();
   std::string Mangled2 = Copy.substr(Copy.size() - 2);
@@ -567,7 +565,7 @@ ParamType lastFuncParamType(const std::string &MangledName) {
 }
 
 // Check if the last argument is signed
-bool isLastFuncParamSigned(const std::string &MangledName) {
+bool isLastFuncParamSigned(StringRef MangledName) {
   return lastFuncParamType(MangledName) == ParamType::SIGNED;
 }
 
@@ -623,7 +621,7 @@ CallInst *mutateCallInst(
   auto NewName = ArgMutate(CI, Args);
   std::string InstName;
   if (!CI->getType()->isVoidTy() && CI->hasName()) {
-    InstName = CI->getName();
+    InstName = CI->getName().str();
     CI->setName(InstName + ".old");
   }
   auto NewCI = addCallInst(M, NewName, CI->getType(), Args, Attrs, CI, Mangle,
@@ -646,13 +644,9 @@ Instruction *mutateCallInst(
   auto Args = getArguments(CI);
   Type *RetTy = CI->getType();
   auto NewName = ArgMutate(CI, Args, RetTy);
-  std::string InstName;
-  if (CI->hasName()) {
-    InstName = CI->getName();
-    CI->setName(InstName + ".old");
-  }
-  auto NewCI = addCallInst(M, NewName, RetTy, Args, Attrs, CI, Mangle,
-                           InstName + ".tmp", TakeFuncName);
+  StringRef InstName = CI->getName();
+  auto NewCI = addCallInst(M, NewName, RetTy, Args, Attrs, CI, Mangle, InstName,
+                           TakeFuncName);
   auto NewI = RetMutate(NewCI);
   NewI->takeName(CI);
   NewI->setDebugLoc(CI->getDebugLoc());
@@ -743,10 +737,10 @@ void makeVector(Instruction *InsPos, std::vector<Value *> &Ops,
 void expandVector(Instruction *InsPos, std::vector<Value *> &Ops,
                   size_t VecPos) {
   auto Vec = Ops[VecPos];
-  auto VT = Vec->getType();
-  if (!VT->isVectorTy())
+  auto *VT = dyn_cast<VectorType>(Vec->getType());
+  if (!VT)
     return;
-  size_t N = VT->getVectorNumElements();
+  size_t N = VT->getNumElements();
   IRBuilder<> Builder(InsPos);
   for (size_t I = 0; I != N; ++I)
     Ops.insert(Ops.begin() + VecPos + I,
@@ -911,11 +905,10 @@ ConstantInt *mapSInt(Module *M, ConstantInt *I, std::function<int(int)> F) {
   return ConstantInt::get(I->getType(), F(I->getSExtValue()), true);
 }
 
-bool isDecoratedSPIRVFunc(const Function *F, std::string *UndecoratedName) {
+bool isDecoratedSPIRVFunc(const Function *F, StringRef &UndecoratedName) {
   if (!F->hasName() || !F->getName().startswith(kSPIRVName::Prefix))
     return false;
-  if (UndecoratedName)
-    *UndecoratedName = undecorateSPIRVFunction(F->getName());
+  UndecoratedName = undecorateSPIRVFunction(F->getName());
   return true;
 }
 
@@ -1051,10 +1044,9 @@ static SPIR::RefParamType transTypeDesc(Type *Ty,
     return SPIR::RefParamType(new SPIR::PrimitiveType(SPIR::PRIMITIVE_FLOAT));
   if (Ty->isDoubleTy())
     return SPIR::RefParamType(new SPIR::PrimitiveType(SPIR::PRIMITIVE_DOUBLE));
-  if (Ty->isVectorTy()) {
-    return SPIR::RefParamType(
-        new SPIR::VectorType(transTypeDesc(Ty->getVectorElementType(), Info),
-                             Ty->getVectorNumElements()));
+  if (auto *VecTy = dyn_cast<VectorType>(Ty)) {
+    return SPIR::RefParamType(new SPIR::VectorType(
+        transTypeDesc(VecTy->getElementType(), Info), VecTy->getNumElements()));
   }
   if (Ty->isArrayTy()) {
     return transTypeDesc(PointerType::get(Ty->getArrayElementType(), 0), Info);
@@ -1081,7 +1073,7 @@ static SPIR::RefParamType transTypeDesc(Type *Ty,
       OS << reinterpret_cast<size_t>(Ty);
       Name = Tmp = std::string("struct_") + OS.str();
     }
-    return SPIR::RefParamType(new SPIR::UserDefinedType(Name));
+    return SPIR::RefParamType(new SPIR::UserDefinedType(Name.str()));
   }
 
   if (Ty->isPointerTy()) {
@@ -1155,15 +1147,12 @@ static SPIR::RefParamType transTypeDesc(Type *Ty,
 Value *getScalarOrArray(Value *V, unsigned Size, Instruction *Pos) {
   if (!V->getType()->isPointerTy())
     return V;
-  assert((isa<ConstantExpr>(V) || isa<GetElementPtrInst>(V)) &&
-         "unexpected value type");
-  auto GEP = cast<User>(V);
+  auto GEP = cast<GEPOperator>(V);
   assert(GEP->getNumOperands() == 3 && "must be a GEP from an array");
-  auto P = GEP->getOperand(0);
-  assert(P->getType()->getPointerElementType()->getArrayNumElements() == Size);
+  assert(GEP->getSourceElementType()->getArrayNumElements() == Size);
   assert(dyn_cast<ConstantInt>(GEP->getOperand(1))->getZExtValue() == 0);
   assert(dyn_cast<ConstantInt>(GEP->getOperand(2))->getZExtValue() == 0);
-  return new LoadInst(P, "", Pos);
+  return new LoadInst(GEP->getSourceElementType(), GEP->getOperand(0), "", Pos);
 }
 
 Constant *getScalarOrVectorConstantInt(Type *T, uint64_t V, bool IsSigned) {
@@ -1171,8 +1160,8 @@ Constant *getScalarOrVectorConstantInt(Type *T, uint64_t V, bool IsSigned) {
     return ConstantInt::get(IT, V);
   if (auto VT = dyn_cast<VectorType>(T)) {
     std::vector<Constant *> EV(
-        VT->getVectorNumElements(),
-        getScalarOrVectorConstantInt(VT->getVectorElementType(), V, IsSigned));
+        VT->getNumElements(),
+        getScalarOrVectorConstantInt(VT->getElementType(), V, IsSigned));
     return ConstantVector::get(EV);
   }
   llvm_unreachable("Invalid type");
@@ -1307,18 +1296,18 @@ Type *getLLVMTypeForSPIRVImageSampledTypePostfix(StringRef Postfix,
 }
 
 std::string getImageBaseTypeName(StringRef Name) {
-  std::string ImageTyName = Name;
 
   SmallVector<StringRef, 4> SubStrs;
   const char Delims[] = {kSPR2TypeName::Delimiter, 0};
   Name.split(SubStrs, Delims);
   if (Name.startswith(kSPR2TypeName::OCLPrefix)) {
-    ImageTyName = SubStrs[1].str();
+    Name = SubStrs[1];
   } else {
-    ImageTyName = SubStrs[0].str();
+    Name = SubStrs[0];
   }
 
-  if (hasAccessQualifiedName(ImageTyName))
+  std::string ImageTyName{Name};
+  if (hasAccessQualifiedName(Name))
     ImageTyName.erase(ImageTyName.size() - 5, 3);
 
   return ImageTyName;
@@ -1339,8 +1328,9 @@ std::string mapOCLTypeNameToSPIRV(StringRef Name, StringRef Acc) {
                       << ", " << Desc.Format << ")\n");
 
     BaseTy = kSPIRVTypeName::Image;
-    OS << getSPIRVImageTypePostfixes(kSPIRVImageSampledTypeName::Void, Desc,
-                                     SPIRSPIRVAccessQualifierMap::map(Acc));
+    OS << getSPIRVImageTypePostfixes(
+        kSPIRVImageSampledTypeName::Void, Desc,
+        SPIRSPIRVAccessQualifierMap::map(Acc.str()));
   } else {
     LLVM_DEBUG(dbgs() << "Mapping of " << Name << " is not implemented\n");
     llvm_unreachable("Not implemented");
@@ -1409,11 +1399,10 @@ manglePipeOrAddressSpaceCastBuiltin(const SPIR::FunctionDescriptor &Fd,
   return SPIR::MANGLE_SUCCESS;
 }
 
-std::string mangleBuiltin(const std::string &UniqName,
-                          ArrayRef<Type *> ArgTypes,
+std::string mangleBuiltin(StringRef UniqName, ArrayRef<Type *> ArgTypes,
                           BuiltinFuncMangleInfo *BtnInfo) {
   if (!BtnInfo)
-    return UniqName;
+    return std::string(UniqName);
   BtnInfo->init(UniqName);
   std::string MangledName;
   LLVM_DEBUG(dbgs() << "[mangle] " << UniqName << " => ");
@@ -1463,31 +1452,53 @@ std::string mangleBuiltin(const std::string &UniqName,
 
 /// Check if access qualifier is encoded in the type Name.
 bool hasAccessQualifiedName(StringRef TyName) {
-  if (TyName.endswith("_ro_t") || TyName.endswith("_wo_t") ||
-      TyName.endswith("_rw_t"))
-    return true;
-  return false;
+  if (TyName.size() < 5)
+    return false;
+  auto Acc = TyName.substr(TyName.size() - 5, 3);
+  return llvm::StringSwitch<bool>(Acc)
+      .Case(kAccessQualPostfix::ReadOnly, true)
+      .Case(kAccessQualPostfix::WriteOnly, true)
+      .Case(kAccessQualPostfix::ReadWrite, true)
+      .Default(false);
+}
+
+SPIRVAccessQualifierKind getAccessQualifier(StringRef TyName) {
+  return SPIRSPIRVAccessQualifierMap::map(
+      getAccessQualifierFullName(TyName).str());
+}
+
+StringRef getAccessQualifierPostfix(SPIRVAccessQualifierKind Access) {
+  switch (Access) {
+  case AccessQualifierReadOnly:
+    return kAccessQualPostfix::ReadOnly;
+  case AccessQualifierWriteOnly:
+    return kAccessQualPostfix::WriteOnly;
+  case AccessQualifierReadWrite:
+    return kAccessQualPostfix::ReadWrite;
+  default:
+    assert(false && "Unrecognized access qualifier!");
+    return kAccessQualPostfix::ReadWrite;
+  }
 }
 
 /// Get access qualifier from the type Name.
-StringRef getAccessQualifier(StringRef TyName) {
+StringRef getAccessQualifierFullName(StringRef TyName) {
   assert(hasAccessQualifiedName(TyName) &&
          "Type is not qualified with access.");
-  auto Acc = TyName.substr(TyName.size() - 4, 2);
+  auto Acc = TyName.substr(TyName.size() - 5, 3);
   return llvm::StringSwitch<StringRef>(Acc)
-      .Case("ro", "read_only")
-      .Case("wo", "write_only")
-      .Case("rw", "read_write")
-      .Default("");
+      .Case(kAccessQualPostfix::ReadOnly, kAccessQualName::ReadOnly)
+      .Case(kAccessQualPostfix::WriteOnly, kAccessQualName::WriteOnly)
+      .Case(kAccessQualPostfix::ReadWrite, kAccessQualName::ReadWrite);
 }
 
 /// Translates OpenCL image type names to SPIR-V.
 Type *getSPIRVImageTypeFromOCL(Module *M, Type *ImageTy) {
   assert(isOCLImageType(ImageTy) && "Unsupported type");
   auto ImageTypeName = ImageTy->getPointerElementType()->getStructName();
-  std::string Acc = kAccessQualName::ReadOnly;
+  StringRef Acc = kAccessQualName::ReadOnly;
   if (hasAccessQualifiedName(ImageTypeName))
-    Acc = getAccessQualifier(ImageTypeName);
+    Acc = getAccessQualifierFullName(ImageTypeName);
   return getOrCreateOpaquePtrType(M, mapOCLTypeNameToSPIRV(ImageTypeName, Acc));
 }
 
@@ -1514,4 +1525,32 @@ bool hasLoopMetadata(const Module *M) {
   return false;
 }
 
+// Returns true if type(s) and number of elements (if vector) is valid
+bool checkTypeForSPIRVExtendedInstLowering(IntrinsicInst *II, SPIRVModule *BM) {
+  switch (II->getIntrinsicID()) {
+  case Intrinsic::fabs:
+  case Intrinsic::ceil: {
+    Type *Ty = II->getType();
+    if (II->getArgOperand(0)->getType() != Ty)
+      return false;
+    int NumElems = 1;
+    if (auto *VecTy = dyn_cast<VectorType>(Ty)) {
+      NumElems = VecTy->getNumElements();
+      Ty = VecTy->getElementType();
+    }
+    if ((!Ty->isFloatTy() && !Ty->isDoubleTy()) ||
+        ((NumElems > 4) && (NumElems != 8) && (NumElems != 16))) {
+      BM->getErrorLog().checkError(false, SPIRVEC_InvalidFunctionCall,
+                                   II->getCalledOperand()->getName().str(), "",
+                                   __FILE__, __LINE__);
+      return false;
+    }
+    break;
+  }
+  default:
+    break;
+  }
+  return true;
+}
+
 } // namespace SPIRV
