@@ -80,11 +80,21 @@ public:
 
   static char ID;
 
+  // This enum sets the mode used to translate the value which is
+  // a function, that is necessary for a convenient function pointers handling.
+  // By default transValue uses 'Decl' mode, which means every function
+  // we meet during the translation should result in its declaration generated.
+  // In 'Pointer' mode we generate OpConstFunctionPointerINTEL constant instead.
+  enum class FuncTransMode { Decl, Pointer };
+
   SPIRVType *transType(Type *T);
   SPIRVType *transSPIRVOpaqueType(Type *T);
 
   SPIRVValue *getTranslatedValue(const Value *) const;
 
+  spv::LoopControlMask getLoopControl(const BranchInst *Branch,
+                                      std::vector<SPIRVWord> &Parameters);
+
   // Translation functions
   bool transAddressingMode();
   bool transAlign(Value *V, SPIRVValue *BV);
@@ -93,6 +103,7 @@ public:
   bool transSourceLanguage();
   bool transExtension();
   bool transBuiltinSet();
+  bool isKnownIntrinsic(Intrinsic::ID Id);
   SPIRVValue *transIntrinsicInst(IntrinsicInst *Intrinsic, SPIRVBasicBlock *BB);
   SPIRVValue *transCallInst(CallInst *Call, SPIRVBasicBlock *BB);
   SPIRVValue *transDirectCallInst(CallInst *Call, SPIRVBasicBlock *BB);
@@ -102,6 +113,7 @@ public:
   bool transDecoration(Value *V, SPIRVValue *BV);
   SPIRVWord transFunctionControlMask(Function *);
   SPIRVFunction *transFunctionDecl(Function *F);
+  void transVectorComputeMetadata(Function *F);
   bool transGlobalVariables();
 
   Op transBoolOpCode(SPIRVValue *Opn, Op OC);
@@ -109,12 +121,17 @@ public:
   // Returns true if succeeds.
   bool translate();
   bool transExecutionMode();
+  void transFPContract();
   SPIRVValue *transConstant(Value *V);
   SPIRVValue *transValue(Value *V, SPIRVBasicBlock *BB,
-                         bool CreateForward = true);
+                         bool CreateForward = true,
+                         FuncTransMode FuncTrans = FuncTransMode::Decl);
   void transGlobalAnnotation(GlobalVariable *V);
-  SPIRVValue *transValueWithoutDecoration(Value *V, SPIRVBasicBlock *BB,
-                                          bool CreateForward = true);
+  SPIRVValue *
+  transValueWithoutDecoration(Value *V, SPIRVBasicBlock *BB,
+                              bool CreateForward = true,
+                              FuncTransMode FuncTrans = FuncTransMode::Decl);
+  void transGlobalIOPipeStorage(GlobalVariable *V, MDNode *IO);
 
   typedef DenseMap<Type *, SPIRVType *> LLVMToSPIRVTypeMap;
   typedef DenseMap<Value *, SPIRVValue *> LLVMToSPIRVValueMap;
@@ -132,6 +149,12 @@ private:
   std::unique_ptr<LLVMToSPIRVDbgTran> DbgTran;
   std::unique_ptr<CallGraph> CG;
 
+  enum class FPContract { UNDEF, DISABLED, ENABLED };
+  DenseMap<Function *, FPContract> FPContractMap;
+  FPContract getFPContract(Function *F);
+  bool joinFPContract(Function *F, FPContract C);
+  void fpContractUpdateRecursive(Function *F, FPContract FPC);
+
   SPIRVType *mapType(Type *T, SPIRVType *BT);
   SPIRVValue *mapValue(Value *V, SPIRVValue *BV);
   SPIRVType *getSPIRVType(Type *T) { return TypeMap[T]; }
@@ -161,11 +184,12 @@ private:
                                SPIRVExtInstSetKind *BuiltinSet = nullptr,
                                SPIRVWord *EntryPoint = nullptr,
                                SmallVectorImpl<std::string> *Dec = nullptr);
-  bool oclIsKernel(Function *F);
-  bool transOCLKernelMetadata();
-  SPIRVInstruction *transBuiltinToInst(const std::string &DemangledName,
-                                       const std::string &MangledName,
-                                       CallInst *CI, SPIRVBasicBlock *BB);
+  bool isKernel(Function *F);
+  bool transMetadata();
+  bool transOCLMetadata();
+  SPIRVInstruction *transBuiltinToInst(StringRef DemangledName, CallInst *CI,
+                                       SPIRVBasicBlock *BB);
+  SPIRVValue *transBuiltinToConstant(StringRef DemangledName, CallInst *CI);
   SPIRVInstruction *transBuiltinToInstWithoutDecoration(Op OC, CallInst *CI,
                                                         SPIRVBasicBlock *BB);
   void mutateFuncArgType(const std::map<unsigned, Type *> &ChangedType,
