@@ -145,22 +145,6 @@ public:
     assert(FuncType && "Invalid func type");
   }
 
-  bool shouldFPContractBeDisabled() const {
-    // We could find some instructions in LLVM IR which look exactly like an
-    // unfused fmuladd. So, we assume that FP_CONTRACT was disabled only if
-    // there are something that looks like uncointracted fmul + fadd, but there
-    // no calls to @llvm.fmuladd.*
-    return FoundUncontractedFMulAdd && !FoundContractedFMulAdd;
-  }
-
-  void setUncontractedFMulAddFound(bool Value = true) {
-    FoundUncontractedFMulAdd = Value;
-  }
-
-  void setContractedFMulAddFound(bool Value = true) {
-    FoundContractedFMulAdd = Value;
-  }
-
 private:
   SPIRVFunctionParameter *addArgument(unsigned TheArgNo, SPIRVId TheId) {
     SPIRVFunctionParameter *Arg = new SPIRVFunctionParameter(
@@ -183,13 +167,37 @@ private:
   std::vector<const SPIRVValue *> Variables;
   typedef std::vector<SPIRVBasicBlock *> SPIRVLBasicBlockVector;
   SPIRVLBasicBlockVector BBVec;
-
-  bool FoundUncontractedFMulAdd = false;
-  bool FoundContractedFMulAdd = false;
 };
 
 typedef SPIRVEntryOpCodeOnly<OpFunctionEnd> SPIRVFunctionEnd;
 
+class SPIRVConstFunctionPointerINTEL : public SPIRVValue {
+  const static Op OC = OpConstFunctionPointerINTEL;
+  const static SPIRVWord FixedWordCount = 4;
+
+public:
+  SPIRVConstFunctionPointerINTEL(SPIRVId TheId, SPIRVType *TheType,
+                                 SPIRVFunction *TheFunction, SPIRVModule *M)
+      : SPIRVValue(M, FixedWordCount, OC, TheType, TheId),
+        TheFunction(TheFunction->getId()) {
+    validate();
+  }
+  SPIRVConstFunctionPointerINTEL()
+      : SPIRVValue(OC), TheFunction(SPIRVID_INVALID) {}
+  SPIRVFunction *getFunction() const { return get<SPIRVFunction>(TheFunction); }
+  _SPIRV_DEF_ENCDEC3(Type, Id, TheFunction)
+  void validate() const override { SPIRVValue::validate(); }
+  SPIRVExtSet getRequiredExtensions() const override {
+    return getSet(ExtensionID::SPV_INTEL_function_pointers);
+  }
+  SPIRVCapVec getRequiredCapability() const override {
+    return getVec(CapabilityFunctionPointersINTEL);
+  }
+
+protected:
+  SPIRVId TheFunction;
+};
+
 } // namespace SPIRV
 
 #endif // SPIRV_LIBSPIRV_SPIRVFUNCTION_H
