@@ -131,19 +131,6 @@ void SPIRVFunctionPointerCallINTEL::validate() const {
   SPIRVFunctionCallGeneric::validate();
 }
 
-SPIRVFunctionPointerINTEL::SPIRVFunctionPointerINTEL(SPIRVId TheId,
-                                                     SPIRVType *TheType,
-                                                     SPIRVFunction *TheFunction,
-                                                     SPIRVBasicBlock *BB)
-    : SPIRVInstruction(FixedWordCount, OC, TheType, TheId, BB),
-      TheFunction(TheFunction->getId()) {
-  validate();
-}
-
-void SPIRVFunctionPointerINTEL::validate() const {
-  SPIRVInstruction::validate();
-}
-
 // ToDo: Each instruction should implement this function
 std::vector<SPIRVValue *> SPIRVInstruction::getOperands() {
   std::vector<SPIRVValue *> Empty;
@@ -183,6 +170,8 @@ bool isSpecConstantOpAllowedOp(Op OC) {
       OpConvertUToPtr,
       OpGenericCastToPtr,
       OpPtrCastToGeneric,
+      OpCrossWorkgroupCastToPtrINTEL,
+      OpPtrCastToCrossWorkgroupINTEL,
       OpBitcast,
       OpQuantizeToF16,
       OpSNegate,
