@@ -294,6 +294,38 @@ _SPIRV_OP(AtomicFlagClear, 319)
 _SPIRV_OP(TypePipeStorage, 322)
 _SPIRV_OP(ConstantPipeStorage, 323)
 _SPIRV_OP(CreatePipeFromPipeStorage, 324)
+_SPIRV_OP(GroupNonUniformElect, 333)
+_SPIRV_OP(GroupNonUniformAll, 334)
+_SPIRV_OP(GroupNonUniformAny, 335)
+_SPIRV_OP(GroupNonUniformAllEqual, 336)
+_SPIRV_OP(GroupNonUniformBroadcast, 337)
+_SPIRV_OP(GroupNonUniformBroadcastFirst, 338)
+_SPIRV_OP(GroupNonUniformBallot, 339)
+_SPIRV_OP(GroupNonUniformInverseBallot, 340)
+_SPIRV_OP(GroupNonUniformBallotBitExtract, 341)
+_SPIRV_OP(GroupNonUniformBallotBitCount, 342)
+_SPIRV_OP(GroupNonUniformBallotFindLSB, 343)
+_SPIRV_OP(GroupNonUniformBallotFindMSB, 344)
+_SPIRV_OP(GroupNonUniformShuffle, 345)
+_SPIRV_OP(GroupNonUniformShuffleXor, 346)
+_SPIRV_OP(GroupNonUniformShuffleUp, 347)
+_SPIRV_OP(GroupNonUniformShuffleDown, 348)
+_SPIRV_OP(GroupNonUniformIAdd, 349)
+_SPIRV_OP(GroupNonUniformFAdd, 350)
+_SPIRV_OP(GroupNonUniformIMul, 351)
+_SPIRV_OP(GroupNonUniformFMul, 352)
+_SPIRV_OP(GroupNonUniformSMin, 353)
+_SPIRV_OP(GroupNonUniformUMin, 354)
+_SPIRV_OP(GroupNonUniformFMin, 355)
+_SPIRV_OP(GroupNonUniformSMax, 356)
+_SPIRV_OP(GroupNonUniformUMax, 357)
+_SPIRV_OP(GroupNonUniformFMax, 358)
+_SPIRV_OP(GroupNonUniformBitwiseAnd, 359)
+_SPIRV_OP(GroupNonUniformBitwiseOr, 360)
+_SPIRV_OP(GroupNonUniformBitwiseXor, 361)
+_SPIRV_OP(GroupNonUniformLogicalAnd, 362)
+_SPIRV_OP(GroupNonUniformLogicalOr, 363)
+_SPIRV_OP(GroupNonUniformLogicalXor, 364)
 _SPIRV_OP(Forward, 1024)
 _SPIRV_OP(SubgroupShuffleINTEL, 5571)
 _SPIRV_OP(SubgroupShuffleDownINTEL, 5572)
@@ -305,11 +337,13 @@ _SPIRV_OP(SubgroupImageBlockReadINTEL, 5577)
 _SPIRV_OP(SubgroupImageBlockWriteINTEL, 5578)
 _SPIRV_OP(SubgroupImageMediaBlockReadINTEL, 5580)
 _SPIRV_OP(SubgroupImageMediaBlockWriteINTEL, 5581)
-_SPIRV_OP(FunctionPointerINTEL, 5600)
+_SPIRV_OP(ConstFunctionPointerINTEL, 5600)
 _SPIRV_OP(FunctionPointerCallINTEL, 5601)
 _SPIRV_OP(AsmTargetINTEL, 5609)
 _SPIRV_OP(AsmINTEL, 5610)
 _SPIRV_OP(AsmCallINTEL, 5611)
+_SPIRV_OP(AssumeTrueINTEL, 5630)
+_SPIRV_OP(ExpectINTEL, 5631)
 _SPIRV_OP(VmeImageINTEL, 5699)
 _SPIRV_OP(TypeVmeImageINTEL, 5700)
 _SPIRV_OP(TypeAvcImePayloadINTEL, 5701)
@@ -434,7 +468,51 @@ _SPIRV_OP(SubgroupAvcSicGetIpeChromaModeINTEL, 5813)
 _SPIRV_OP(SubgroupAvcSicGetPackedSkcLumaCountThresholdINTEL, 5814)
 _SPIRV_OP(SubgroupAvcSicGetPackedSkcLumaSumThresholdINTEL, 5815)
 _SPIRV_OP(SubgroupAvcSicGetInterRawSadsINTEL, 5816)
+_SPIRV_OP(ArbitraryFloatSinCosPiINTEL, 5840)
+_SPIRV_OP(ArbitraryFloatCastINTEL, 5841)
+_SPIRV_OP(ArbitraryFloatCastFromIntINTEL, 5842)
+_SPIRV_OP(ArbitraryFloatCastToIntINTEL, 5843)
+_SPIRV_OP(ArbitraryFloatAddINTEL, 5846)
+_SPIRV_OP(ArbitraryFloatSubINTEL, 5847)
+_SPIRV_OP(ArbitraryFloatMulINTEL, 5848)
+_SPIRV_OP(ArbitraryFloatDivINTEL, 5849)
+_SPIRV_OP(ArbitraryFloatGTINTEL, 5850)
+_SPIRV_OP(ArbitraryFloatGEINTEL, 5851)
+_SPIRV_OP(ArbitraryFloatLTINTEL, 5852)
+_SPIRV_OP(ArbitraryFloatLEINTEL, 5853)
+_SPIRV_OP(ArbitraryFloatEQINTEL, 5854)
+_SPIRV_OP(ArbitraryFloatRecipINTEL, 5855)
+_SPIRV_OP(ArbitraryFloatRSqrtINTEL, 5856)
+_SPIRV_OP(ArbitraryFloatCbrtINTEL, 5857)
+_SPIRV_OP(ArbitraryFloatHypotINTEL, 5858)
+_SPIRV_OP(ArbitraryFloatSqrtINTEL, 5859)
+_SPIRV_OP(ArbitraryFloatLogINTEL, 5860)
+_SPIRV_OP(ArbitraryFloatLog2INTEL, 5861)
+_SPIRV_OP(ArbitraryFloatLog10INTEL, 5862)
+_SPIRV_OP(ArbitraryFloatLog1pINTEL, 5863)
+_SPIRV_OP(ArbitraryFloatExpINTEL, 5864)
+_SPIRV_OP(ArbitraryFloatExp2INTEL, 5865)
+_SPIRV_OP(ArbitraryFloatExp10INTEL, 5866)
+_SPIRV_OP(ArbitraryFloatExpm1INTEL, 5867)
+_SPIRV_OP(ArbitraryFloatSinINTEL, 5868)
+_SPIRV_OP(ArbitraryFloatCosINTEL, 5869)
+_SPIRV_OP(ArbitraryFloatSinCosINTEL, 5870)
+_SPIRV_OP(ArbitraryFloatSinPiINTEL, 5871)
+_SPIRV_OP(ArbitraryFloatCosPiINTEL, 5872)
+_SPIRV_OP(ArbitraryFloatASinINTEL, 5873)
+_SPIRV_OP(ArbitraryFloatASinPiINTEL, 5874)
+_SPIRV_OP(ArbitraryFloatACosINTEL, 5875)
+_SPIRV_OP(ArbitraryFloatACosPiINTEL, 5876)
+_SPIRV_OP(ArbitraryFloatATanINTEL, 5877)
+_SPIRV_OP(ArbitraryFloatATanPiINTEL, 5878)
+_SPIRV_OP(ArbitraryFloatATan2INTEL, 5879)
+_SPIRV_OP(ArbitraryFloatPowINTEL, 5880)
+_SPIRV_OP(ArbitraryFloatPowRINTEL, 5881)
+_SPIRV_OP(ArbitraryFloatPowNINTEL, 5882)
 _SPIRV_OP(LoopControlINTEL, 5887)
+_SPIRV_OP(PtrCastToCrossWorkgroupINTEL, 5934)
+_SPIRV_OP(CrossWorkgroupCastToPtrINTEL, 5938)
 _SPIRV_OP(ReadPipeBlockingINTEL, 5946)
 _SPIRV_OP(WritePipeBlockingINTEL, 5947)
 _SPIRV_OP(FPGARegINTEL, 5949)
+_SPIRV_OP(TypeBufferSurfaceINTEL, 6086)
