@@ -169,6 +169,13 @@ const SPIRVDecoder &operator>>(const SPIRVDecoder &I, std::vector<T> &V) {
   return I;
 }
 
+template <typename T>
+const SPIRVDecoder &operator>>(const SPIRVDecoder &I, llvm::Optional<T> &V) {
+  if (V)
+    I >> V.getValue();
+  return I;
+}
+
 template <typename T>
 const SPIRVEncoder &operator<<(const SPIRVEncoder &O, T V) {
 #ifdef _SPIRV_SUPPORT_TEXT_FMT
@@ -194,6 +201,14 @@ const SPIRVEncoder &operator<<(const SPIRVEncoder &O, const std::vector<T> &V) {
   return O;
 }
 
+template <typename T>
+const SPIRVEncoder &operator<<(const SPIRVEncoder &O,
+                               const llvm::Optional<T> &V) {
+  if (V)
+    O << V.getValue();
+  return O;
+}
+
 template <typename IterTy>
 const SPIRVEncoder &operator<<(const SPIRVEncoder &Encoder,
                                const std::pair<IterTy, IterTy> &Range) {
